/*
 * Copyright (C) 2005 oudot@rstack.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

 /*
  * This source code is freely provided for you to be able to improve the
  * stealthiness of your WLAN networks. This is just a proof of concept.
  * Use it at your own risks. It was coded for a Linksys WRT54GS with
  * the GPL firmware OpenWRT (openwrt.org). I apologize for the coding style
  * bugs & errors that might be there. I just needed an easy proof of concept.
  *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>

#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include <time.h>

#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/wireless.h>

#include <getopt.h>

#include <pcap.h>

#define MAX_SSID_LEN 32
#define PRISM 144

char ssid_name[MAX_SSID_LEN+1];
u_char wep[13];
char *my_essid;
char *interface;

int get_ssid(const u_char *packet_data){
  if(packet_data[24] == 0x00 && packet_data[26] != '\0'){
    u_short max_len;
    int i;

    max_len = packet_data[25] > MAX_SSID_LEN ? MAX_SSID_LEN : packet_data[25];

    memset(ssid_name, 0, MAX_SSID_LEN+1);
    memcpy(ssid_name, &packet_data[26], max_len);

	for(i=0; i<max_len; i++)
		if( ssid_name[i] < ' ')
			ssid_name[i] = '.';

    return 0;
  }
  return -1;
}



void packet_handler(u_char *user, const struct pcap_pkthdr *pkthdr,
    const u_char *packet_data)
{
  char exe[70];
  memset(exe, 0, 70+1);
  if(packet_data[0+PRISM] == 0x40){
    if(get_ssid(packet_data+PRISM)==-1)
	    return 0;
    if(strlen(ssid_name)==0)
	    return 0;

    fprintf(stderr,".", ssid_name);

    if(!strncmp(ssid_name,my_essid,strlen(ssid_name))){
	int i;
	fprintf(stderr,"Probe Request (%s)\n", ssid_name);
	memset(wep, 0, 13);
	memcpy(wep, &packet_data[PRISM+9], 6);
	for(i=0;i<13;i++)
		fprintf(stderr,"%.2x", wep[i]);
	fprintf(stderr,"\n nvram set wl0_key1=%.2x%.2x%.2x%.2x%.2x\n",wep[1],wep[2],wep[3],wep[4],wep[5]);
	sprintf(exe,"nvram set wl0_key1=%.2x%.2x%.2x%.2x%.2x",wep[1],wep[2],wep[3],wep[4],wep[5]);
	system(exe);
	sprintf(exe,"wlconf %s down",interface);
	system(exe);
	sprintf(exe,"wlconf %s up",interface);
	system(exe);

	fprintf(stderr,"\n");
	if(strlen(interface)+37 < 70){
		sprintf(exe,"wl monitor 0 && iwconfig %s mode master",interface);
		system(exe);
		fprintf(stderr,"Mode MASTER (%s)\n",exe);
		sleep(60);
		memset(exe, 0, 70+1);
		sprintf(exe,"wl monitor 1 && iwconfig %s mode managed",interface);
		system(exe);
		fprintf(stderr,"Mode MONITOR (%s)\n",exe);
	  }
    }
  }
  else
	 fprintf(stderr,"");
}


int main(int argc, char **argv){
  pcap_t *pcap;
  char err[PCAP_ERRBUF_SIZE];
  char exe[70];
  u_char *p;

  memset(exe, 0, 70+1);
  if(argc==1){
	sprintf(exe,"%s `nvram get wl0_ifname` `nvram get wl0_ssid`", argv[0]);
	system(exe);
	exit(1);
  }
  interface = argv[1];
  if(argc==3){
    my_essid=argv[2];
  } else {
    my_essid="network";
  }

  fprintf(stderr,"Seeking for [%s]\n", my_essid);

  sprintf(exe,"wl monitor 1 && iwconfig %s mode managed",interface);
  system(exe);
  fprintf(stderr,"Mode MONITOR (%s)\n",exe);

  pcap = pcap_open_live(interface, 0xffff, 1, 1, err);
  if(pcap == NULL){
	fprintf(stderr,"pcap_open_live error: %s\n", err);
	exit(1);
  }

  for(;;){
	pcap_loop(pcap, 1, packet_handler, (u_char*)p);
  }
}
