/*
 * Copyright (C) 2005 oudot@rstack.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */


/*
 * This source code is freely provided for you to be able to improve the
 * stealthiness of your WLAN networks. This is just a proof of concept.
 * Use it at your own risks. It was coded for a Linksys WRT54GS with
 * the GPL firmware OpenWRT (openwrt.org). I apologize for the coding style
 * bugs & errors that might be there. I just needed an easy proof of concept.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>

#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include <time.h>

#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/wireless.h>

#include <getopt.h>

#include <pcap.h>

#define MAX_SSID_LEN 32
#define PRISM 144
#define VERSION "1.0"

char ssid_name[MAX_SSID_LEN+1];
char *my_essid;
char *interface;
char *prog;
int seconds=20;
time_t now;
pcap_t *pcap;
u_char *p;

int get_ssid(const u_char *packet_data){
  memset(ssid_name, 0, MAX_SSID_LEN+1);
  if(packet_data[24] == 0x00 && packet_data[26] != '\0'){
    u_short max_len;
    int i;

    max_len = packet_data[25] > MAX_SSID_LEN ? MAX_SSID_LEN : packet_data[25];

    memcpy(ssid_name, &packet_data[26], max_len);

	for(i=0; i<max_len; i++)
		if( ssid_name[i] < ' ')
			ssid_name[i] = '.';

    return 0;
  }
  return -1;
}

void invisible()
{
  char exe[70];
  memset(exe, 0, 70+1);
  time(&now);
  sprintf(exe,"wl monitor 1 && iwconfig %s mode managed",interface);
  system(exe);
  fprintf(stderr,"[*] %s   Mode STEALTH - waiting for probe request - no beacon sent\n    (console)# %s\n",ctime(&now),exe);
}

void visible()
{
  char exe[70];
  memset(exe, 0, 70+1);
  time(&now);
  sprintf(exe,"wl monitor 0 && iwconfig %s mode master",interface);
  system(exe);
  fprintf(stderr,"\n[ ] %s   Mode MASTER - will switch to stealth mode in %d seconds\n    (console)# %s\n",ctime(&now),seconds,exe);
}

int packet_handler(u_char *user, const struct pcap_pkthdr *pkthdr,
    const u_char *packet_data)
{
  if(packet_data[0+PRISM] == 0x40){

    if(get_ssid(packet_data+PRISM)==-1)
      return 0;

    if(strlen(ssid_name)==0)
      return 0;

	/* One dot written for any Probe Request */
	fprintf(stderr,".");

	if(!strncmp(ssid_name,my_essid,strlen(ssid_name))){
	    u_char src[7];
	    memset(src,0,7);
	    memcpy(src,&packet_data[PRISM+10],6);

	    fprintf(stderr,"\n   OK Probe request for \"%s\" sent from %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n", ssid_name,src[0],src[1],src[2],src[3],src[4],src[5]);

	    visible();
	    sleep(seconds);
	    stop_capture(pcap);
	    invisible();
	    capture();
	}
    }
  return 0;
}

int stop_capture()
{
	pcap_close(pcap);
	return 0;
}
int capture()
{
  char err[PCAP_ERRBUF_SIZE];
  pcap = pcap_open_live(interface, 0xffff, 1, 1, err);
  if(pcap == NULL){
	fprintf(stderr,"pcap_open_live error: %s\n", err);
	exit(1);
  }
  for(;;){
	pcap_dispatch(pcap, 1, packet_handler, (u_char*)p);
  }
  return 0;
}

int main(int argc, char **argv){
  char exe[70];
  memset(exe, 0, 70+1);

  if(argc==1){
	sprintf(exe,"%s `nvram get wl0_ifname` `nvram get wl0_ssid`", argv[0]);
	system(exe);
	exit(1);
  }
  fprintf(stderr,"\n<< WKNOCK %s - Adding knocking mode for stealth WLan >>\n   http://rstack.org/oudot/wknock    oudot@rstack.org\n\n",VERSION);
  interface = argv[1];
  if(argc==3){
    my_essid=argv[2];
  }
  prog = argv[0];

  fprintf(stderr,"Switching to stealth mode for essid \"%s\"\n", my_essid);

  invisible();
  capture();
}


